#ifndef SJTU_MAP_SUBTASK2_HPP
#define SJTU_MAP_SUBTASK2_HPP
#include <iostream>
#include <type_traits>
#include "utility.hpp"
namespace sjtu {
template <typename _Tp>
static constexpr bool can_be_used_in_cout_v = requires(std::ostream &os, const _Tp &dat) {
  os << dat;
};
template <typename _Tp>
static constexpr bool is_pair_v = false;
template <typename _T1, typename _T2>
static constexpr bool is_pair_v<std::pair<_T1, _T2> > = true;
template <typename _T1, typename _T2>
static constexpr bool is_pair_v<sjtu::pair<_T1, _T2> > = true;
template <typename _Tp>
static constexpr bool is_container_v = requires(const _Tp &x) {
  std::begin(x);
  std::end(x);
};
template <typename _Tp>
static constexpr bool is_container_s_container_v = requires(const _Tp &x) {
  std::begin(*std::begin(x));
  std::end(*std::begin(x));
};
template <typename _Tp>
void print(const _Tp &dat, size_t indent_length = 0) {
  static constexpr bool need_indent = sjtu::is_container_s_container_v<_Tp>;
  if constexpr (sjtu::can_be_used_in_cout_v<_Tp>) {
    if constexpr (std::is_bounded_array_v<_Tp>) {
      for (size_t i = 0; i < indent_length; i++) std::cout << ' ';
      std::cout << '[';
      if constexpr (need_indent) std::cout << '\n';
      for (size_t i = 0; i < std::extent_v<_Tp>; i++) {
        if (need_indent == false && i) std::cout << ' ';
        print(dat[i], indent_length + 1);
        if constexpr (need_indent) std::cout << '\n';
      }
      if constexpr (need_indent)
        for (size_t i = 0; i < indent_length; i++) std::cout << ' ';
      std::cout << ']';
    } else
      std::cout << dat;
  } else if constexpr (sjtu::is_container_v<_Tp>) {
    for (size_t i = 0; i < indent_length; i++) std::cout << ' ';
    std::cout << '[';
    if constexpr (need_indent) std::cout << '\n';
    for (auto it = std::begin(dat); it != std::end(dat); it++) {
      if (need_indent == false && it != std::begin(dat)) std::cout << ' ';
      print(*it, indent_length + 1);
      if constexpr (need_indent) std::cout << '\n';
    }
    if constexpr (need_indent)
      for (size_t i = 0; i < indent_length; i++) std::cout << ' ';
    std::cout << ']';
  } else if constexpr (sjtu::is_pair_v<_Tp>) {
    std::cout << '(';
    print(dat.first, indent_length);
    std::cout << ',';
    print(dat.second, indent_length);
    std::cout << ')';
  } else
    static_assert(sjtu::can_be_used_in_cout_v<_Tp>, "Cannot print this type.");
}
}  // namespace sjtu
#endif